/*
 * Perform SPI communication using the JZ4780 SPI peripheral.
 *
 * Copyright (C) 2023, 2024 Paul Boddie <paul@boddie.org.uk>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA  02110-1301, USA
 */

#pragma once

#include <l4/re/c/dma_space.h>
#include <l4/sys/types.h>
#include <stdint.h>



#ifdef __cplusplus

#include <l4/devices/dma-jz4780.h>
#include <l4/devices/spi-common.h>

/* SPI peripheral channel. */

class Spi_jz4780_channel : public Spi_channel
{
public:
  explicit Spi_jz4780_channel(l4_addr_t spi_start, l4_addr_t start,
                              enum Clock_identifiers clock,
                              Cpm_chip *cpm,
                              Dma_channel *dma,
                              enum Dma_jz4780_request_type request_type,
                              uint64_t frequency);
};

/* SPI peripheral. */

class Spi_jz4780_chip : public Spi_chip
{
protected:
  unsigned int num_channels()
  { return 2; }

  Spi_channel *_get_channel(uint8_t channel, Dma_channel *dma,
                            uint64_t frequency);

public:
  explicit Spi_jz4780_chip(l4_addr_t spi_start, l4_addr_t start, l4_addr_t end, Cpm_chip *cpm);
};

#endif /* __cplusplus */



/* C language interface. */

EXTERN_C_BEGIN

void *jz4780_spi_init(l4_addr_t spi_start, l4_addr_t start, l4_addr_t end,
                      void *cpm);

void *jz4780_spi_get_channel(void *spi, uint8_t channel, void *dma, uint64_t frequency);

uint32_t jz4780_spi_send(void *channel, uint32_t bytes, const uint8_t data[]);

uint32_t jz4780_spi_send_dc(void *channel, uint32_t bytes, const uint8_t data[],
                            const int dc[], uint8_t char_size, int big_endian);

uint32_t jz4780_spi_send_units(void *channel, uint32_t bytes, const uint8_t data[],
                               uint8_t unit_size, uint8_t char_size, int big_endian);

uint32_t jz4780_spi_transfer(void *channel, l4_addr_t vaddr,
                             l4re_dma_space_dma_addr_t paddr, uint32_t count,
                             uint8_t unit_size, uint8_t char_size);

uint32_t jz4780_spi_transfer_descriptor(void *channel, l4_addr_t vaddr,
                                        l4re_dma_space_dma_addr_t paddr,
                                        uint32_t count, uint8_t unit_size,
                                        uint8_t char_size, l4_addr_t desc_vaddr,
                                        l4re_dma_space_dma_addr_t desc_paddr);

EXTERN_C_END
